/**
 * UI scripting code.
 * 
 * @author    Andy Frank
 * @creation  27 Dec 04
 */

////////////////////////////////////////////////////////////////
// Global
////////////////////////////////////////////////////////////////

var collect = null;

////////////////////////////////////////////////////////////////
// Lifecycle
////////////////////////////////////////////////////////////////

var init = false;
function startup()
{
  // This method was getting called twice, not sure why
  // so short circuit if method already invoked.
  if (init) return;
  refreshModel();    
  init = true;
}

function refreshModel()
{
  loadFeedModel();  
  model.sort();
  var tree = document.getElementById("feedTree");  
  tree.view = new FeedTreeModel();
}

var clean = false;
function cleanup()
{
  if (clean) return;
  saveFeedModel();
  clean = true;
}

////////////////////////////////////////////////////////////////
// Commands
////////////////////////////////////////////////////////////////

/**
 * Check feeds for new items.
 */
function checkFeeds()
{
  doCheckFeeds();  
}

/**
 * Check an indiviual feed.
 */
function checkSoloFeed()
{
  if (checkInProgress) return;  
  var tree = document.getElementById("feedTree");
  var elem = document.getElementById("busyAnimation");
  elem.style.visibility = "visible";
  cancelCheck = true; // force cancel after this feed updates
  precheckFeed(tree.currentIndex);  
}

/**
 * Add a new feed.
 */
function addFeed()
{
  var params = { url:"" };
  var win = window.openDialog("chrome://newsfox/content/addFeed.xul",
    "newsfox-dialog","chrome,centerscreen,modal", params);  
  // TODO - Need to verify URL first
  createNewFeed(model, params.url);  
  saveFeedModel();
  refreshModel();   
}

/**
 * Permantly deleted a feed.
 */
function deleteFeed()
{
  var tree = document.getElementById("feedTree");
  var feed = model.get(tree.currentIndex);
  
  if (confirm("Are you sure you want to permanently delete this " +
    "feed and all articles?\n" + feed.getDisplayName()))
  {
    deleteFeedFromDisk(feed);
    model.remove(feed);
    saveFeedModel();
    refreshModel();
    // Tree loses selection when node removed - need to force
    // next/prev node selected b/f calling this method.
    //feedSelected();
  }
}

/**
 * Restore an deleted article.
 */
function restoreArticle()
{
  /*
  var tree = document.getElementById("feedTree");
  var index = tree.currentIndex;
  if (index != model.size()+1) return;
  
  tree = document.getElementById("articleTree");
  var start = new Object();
  var end   = new Object();
  var range = tree.view.selection.getRangeCount();
  
  for (var i=0; i<range; i++)
  {
    tree.view.selection.getRangeAt(i,start,end);
    for (var j=start.value; j<=end.value; j++)
    {
      var art = collect.get(j);
      art.trash = false;
    }    
    collect = new TrashCollection();
    tree.treeBoxObject.rowCountChanged(start.value, -(end.value-start.value+1));
  }
  
  // Update unread count
  tree = document.getElementById("feedTree");
  tree.treeBoxObject.invalidate();   
  */
}

/**
 * Delete an article.
 */
function deleteArticle()
{  
  try
  {
    var tree = document.getElementById("feedTree");
    var index = tree.currentIndex;
    var feed = model.get(index);
  
    tree = document.getElementById("articleTree");
    var start = new Object();
    var end   = new Object();
    var range = tree.view.selection.getRangeCount();
    if (range == 0) return;
    
    if (!confirm("Are you sure you wish to permanently delete these articles?"))
      return;
  
    // Regular delete
    for (var i=0; i<range; i++)
    {
      tree.view.selection.getRangeAt(i,start,end);
      for (var j=start.value; j<=end.value; j++)
        feed.remove(start.value);
        //feed.remove(j);

      //collect = new NormalCollection(feed);
      //noToggleOrder = true;
      //tree.view.cycleHeader(lastColName, lastCol);
      //noToggleOrder = false;      
      //tree.treeBoxObject.rowCountChanged(start.value, -(end.value-start.value+1));
    }
    
    saveFeed(feed);
    saveFeedModel();
    
    // Update unread count
    tree = document.getElementById("feedTree");
    tree.treeBoxObject.invalidate();
    feedSelected();
  }
  catch (err)
  {
    var msg = "deleteArticle(): " + err;
    alert(msg);
  }
}

/**
 * Show help.
 */
function help()
{
  frames["content"].location.href = "chrome://newsfox/content/help/start.html";
}

/**
 * Troubleshoot feed errors.
 */
function troubleshoot()
{
  var tree = document.getElementById("feedTree");
  var feed = model.get(tree.currentIndex);
  
  var iframe = document.getElementById("content");
  var doc = iframe.contentDocument;
  doc.body.style.font = "10pt Verdana";
  doc.body.style.background = "white";
  while (doc.body.childNodes.length > 0)
    doc.body.removeChild(doc.body.childNodes[0]);  
  
  var summary = getErrorSummary(feed.error);
  var remedies = getErrorRemedies(feed.error);
  
  var b = doc.createElement("b");
  b.appendChild(doc.createTextNode(summary));
  
  var p = doc.createElement("p");
  p.innerHTML = remedies;
  
  doc.body.appendChild(b);
  doc.body.appendChild(doc.createElement("hr"));
  doc.body.appendChild(p);
}

////////////////////////////////////////////////////////////////
// Util
////////////////////////////////////////////////////////////////

function getParentFeed(art)
{
  for (var i=0; i<model.size(); i++)
  {
    var feed = model.get(i);      
    for (var j=0; j<feed.size(); j++)
      if (feed.get(j).link == art.link)
        return feed;
  }
  return null;
}

////////////////////////////////////////////////////////////////
// Selection
////////////////////////////////////////////////////////////////

/**
 * Feed selected.
 */
function feedSelected()
{
  var tree = document.getElementById("feedTree");
  var index = tree.currentIndex;  

  var feed = model.get(index);
  if (!feed.loaded)
  {
    loadFeed(feed);
    feed.loaded = true;
  }
  
  if (index < model.size()) collect = new NormalCollection(feed);
  else if (index == model.size()) collect = new FlaggedCollection();
  else collect = new TrashCollection();
  
  tree = document.getElementById("articleTree");  
  tree.view = null;  
  tree.view = new ArticleTreeModel();
  if (lastCol != null) 
  {
    noToggleOrder = true;
    tree.view.cycleHeader(lastColName, lastCol);
    noToggleOrder = false;
  }
}

/**
 * Article selected.
 */
function articleSelected()
{
  var tree = document.getElementById("feedTree");
  var feedIndex = tree.currentIndex;
  tree = document.getElementById("articleTree");
  var index = tree.currentIndex;
  var art = collect.get(index);
  var read = collect.feed.isRead(art);
  
  if (!read)
  {
    collect.feed.setRead(art, true);
    tree.treeBoxObject.invalidateRow(index);
    var tree = document.getElementById("feedTree");
    tree.treeBoxObject.invalidateRow(feedIndex);
  }
  
  // Display body in content
  var iframe = document.getElementById("content");
  var doc = iframe.contentDocument;
  doc.body.style.font = "10pt Verdana";
  doc.body.style.background = "white";
  while (doc.body.childNodes.length > 0)
    doc.body.removeChild(doc.body.childNodes[0]);
  
  var title = doc.createElement("b");
  title.innerHTML = art.title;
  
  var link = doc.createElement("a");
  link.href = art.link;
  link.innerHTML = art.link;
  
  var heading = doc.createElement("div");
  heading.style.background = "#e3dfd9"; //"#d4d0c8";
  heading.style.padding = "10px";
  heading.style.overflow = "hidden";
  //heading.style.MozBorderRadius = "6px";

  heading.appendChild(title);
  heading.appendChild(doc.createElement("br"));
  heading.appendChild(doc.createTextNode(formatDate(art.date)));
  heading.appendChild(doc.createElement("br"));
  heading.appendChild(link);
  doc.body.appendChild(heading);
  
  doc.body.appendChild(doc.createElement("p"));
  
  var body = doc.createElement("div");
  body.innerHTML = art.body;
  doc.body.appendChild(body);  
}

////////////////////////////////////////////////////////////////
// Util
////////////////////////////////////////////////////////////////

function formatDate(date)
{
  var hour = date.getHours();
  var m = "AM";
  if (hour == 0) hour = 12;
  else if (hour == 12) m = "PM";
  else if (hour > 12) { hour -= 12; m = "PM"; }  
  
  var min = "" + date.getMinutes();
  if (min.length == 1) min = "0" + min;
  
  var v = (date.getMonth()+1) + "/" + date.getDate() + "/" + date.getFullYear();
  v += " " + hour + ":" + min + " " + m; 
  // TODO - timezone?
  //v += " " + (date.getTimezoneOffset() / 60) + "H";
  return v;
}

////////////////////////////////////////////////////////////////
// FeedTreeModel
////////////////////////////////////////////////////////////////

function FeedTreeModel() 
{
  var iconOk  = "chrome://newsfox/skin/images/feed.png";
  var iconErr = "chrome://newsfox/skin/images/brokenFeed.png";
  var flagIcon = "chrome://newsfox/skin/images/flagFeed.png";
  var trashIcon = "chrome://newsfox/skin/images/trash.png";
  
  // 3/25/05 - Temp disable flag/trash folder
  //this.rowCount = model.size() + 2;
  this.rowCount = model.size();
  this.getCellText = function(row,col)
  {
    if (row < model.size())
    {
      var feed = model.get(row);
      var text = feed.getDisplayName();
      var unread = feed.getUnread();
      if (unread > 0) text += " (" + unread + ")";
      return text;
    }
    else if (row - model.size() == 0) return "Flagged Items";
    else return "Trash"; 
  }
  this.setTree = function(treebox){ this.treebox = treebox; }
  this.isContainer = function(row){ return false; }
  this.isSeparator = function(row){ return false; }
  this.isSorted = function(row){ return false; }
  this.getLevel = function(row){ return 0; }
  this.getImageSrc = function(row,col) 
  { 
    if (row < model.size())
      return (model.get(row).error == ERROR_OK) ? iconOk : iconErr; 
    else if (row - model.size() == 0) return flagIcon;
    else return trashIcon;
  }
  this.getRowProperties = function(row,props) {}
  this.getCellProperties = function(row,col,props) 
  {
    if (row >= model.size()) return;
    if (model.get(row).getUnread() > 0)
    {
      var aserv = Components.classes["@mozilla.org/atom-service;1"].
        getService(Components.interfaces.nsIAtomService);
      props.AppendElement(aserv.getAtom("unread"));
    }
  }
  this.getColumnProperties = function(colid,col,props) {}
}

////////////////////////////////////////////////////////////////
// ArticleTreeModel
////////////////////////////////////////////////////////////////

var lastCol = null;
var lastColName = null;
var noToggleOrder = false;

function ArticleTreeModel() 
{
  var flagIcon = "chrome://newsfox/skin/images/flag.png";
  var readIcon = "chrome://newsfox/skin/images/read.png";
  var unreadIcon = "chrome://newsfox/skin/images/unread.png";  
  
  this.rowCount = collect.size();
  this.getCellText = function(row,col)
  {
    switch (col)
    {
      case "title": return collect.get(row).title;
      case "date":  return formatDate(collect.get(row).date);
      default: return "";
    }
  }
  this.setTree = function(treebox){ this.treebox = treebox; }
  this.isContainer = function(row){ return false; }
  this.isSeparator = function(row){ return false; }
  this.isSorted = function(row){ return false; }
  this.getLevel = function(row){ return 0; }
  this.getImageSrc = function(row,col)
  { 
    var read = collect.feed.isRead(collect.get(row));
    var flag = collect.feed.isFlagged(collect.get(row));
    switch (col)
    {
      case "read": return read ? readIcon : unreadIcon;
      case "flag": return flag ? flagIcon : readIcon;
      default: return null;
    }
  }
  this.getRowProperties = function(row,props) {}
  this.getCellProperties = function(row,col,props) 
  {
    if (!collect.feed.isRead(collect.get(row)))
    {
      var aserv = Components.classes["@mozilla.org/atom-service;1"].
        getService(Components.interfaces.nsIAtomService);
      props.AppendElement(aserv.getAtom("unread"));
    }
  }
  this.getColumnProperties = function(colid,col,props) {}
  this.cycleHeader = function(col,elem)
  {
    /* TODO - disabled for now
    
    // Don't support sorting of flag/read yet
    if (col != "title" && col != "date") return;
    
    var array = collect.getArray();
    if (noToggleOrder)
    {
      var ascend = lastCol.getAttribute("sortDirection") == "ascending";
    }
    else
    {
      var ascend = true;    
      if (lastCol == elem && lastCol.getAttribute("sortDirection") == "ascending")
        ascend = false;
    }
    
    // TODO - Simple bubble sort, use
    // more effecient algoritm
    
    for (var i=0; i<array.length-1; i++)
      for (var j=array.length-1; j>i; j--)
      {
        if (col == "title")
        {
          var a = array[j].title.toLowerCase();
          var b = array[j-1].title.toLowerCase();
        }
        else
        {
          var a = array[j].date;
          var b = array[j-1].date;
        }
             
        if ((ascend && (a < b)) || (!ascend && (a > b)))
        {
          var temp = array[j];
          array[j] = array[j-1];
          array[j-1] = temp;          
        }
      }
    
    if (lastCol != null) lastCol.setAttribute("sortDirection", "natural");
    elem.setAttribute("sortDirection", ascend ? "ascending" : "descending");
    lastCol = elem;
    lastColName = col;
    
    var tree = document.getElementById("articleTree");
    tree.treeBoxObject.invalidate();
    */
  }
  this.cycleCell = function(row,col)
  {
    var art = collect.get(row);
    if (col == "read") 
    {
      var read = collect.feed.isRead(art);
      collect.feed.setRead(art, !read);      
      var tree = document.getElementById("articleTree");
      tree.treeBoxObject.invalidateRow(row);      
      tree = document.getElementById("feedTree");
      //tree.treeBoxObject.invalidateRow(index);  
      tree.treeBoxObject.invalidate();    
    }
    else if (col == "flag")
    {
      //collect.get(row).flag = !collect.get(row).flag;
      var flag = collect.feed.isFlagged(art);
      collect.feed.setFlagged(art, !flag);      
      var tree = document.getElementById("articleTree");
      //tree.treeBoxObject.invalidateRow(row);      
      tree.treeBoxObject.invalidate();
    }
  }
}

